# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: http.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nhttp.proto\x12\x12KismetExternalHttp\"\x16\n\x14HttpAuthTokenRequest\"\x1e\n\rHttpAuthToken\x12\r\n\x05token\x18\x01 \x02(\t\"N\n\x0fHttpRegisterUri\x12\x0b\n\x03uri\x18\x01 \x02(\t\x12\x0e\n\x06method\x18\x02 \x02(\t\x12\x1e\n\x16obsolete_auth_required\x18\x03 \x01(\x08\"5\n\x13SubHttpVariableData\x12\r\n\x05\x66ield\x18\x01 \x02(\t\x12\x0f\n\x07\x63ontent\x18\x02 \x02(\t\"z\n\x0bHttpRequest\x12\x0e\n\x06req_id\x18\x01 \x02(\r\x12\x0b\n\x03uri\x18\x02 \x02(\t\x12\x0e\n\x06method\x18\x03 \x02(\t\x12>\n\rvariable_data\x18\x04 \x03(\x0b\x32\'.KismetExternalHttp.SubHttpVariableData\"#\n\x11HttpRequestCancel\x12\x0e\n\x06req_id\x18\x01 \x02(\r\"0\n\rSubHttpHeader\x12\x0e\n\x06header\x18\x01 \x02(\t\x12\x0f\n\x07\x63ontent\x18\x02 \x02(\t\"\x96\x01\n\x0cHttpResponse\x12\x0e\n\x06req_id\x18\x01 \x02(\r\x12\x39\n\x0eheader_content\x18\x02 \x03(\x0b\x32!.KismetExternalHttp.SubHttpHeader\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\x12\x12\n\nresultcode\x18\x04 \x01(\r\x12\x16\n\x0e\x63lose_response\x18\x05 \x01(\x08\x42\x02H\x03')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'http_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\003'
  _HTTPAUTHTOKENREQUEST._serialized_start=34
  _HTTPAUTHTOKENREQUEST._serialized_end=56
  _HTTPAUTHTOKEN._serialized_start=58
  _HTTPAUTHTOKEN._serialized_end=88
  _HTTPREGISTERURI._serialized_start=90
  _HTTPREGISTERURI._serialized_end=168
  _SUBHTTPVARIABLEDATA._serialized_start=170
  _SUBHTTPVARIABLEDATA._serialized_end=223
  _HTTPREQUEST._serialized_start=225
  _HTTPREQUEST._serialized_end=347
  _HTTPREQUESTCANCEL._serialized_start=349
  _HTTPREQUESTCANCEL._serialized_end=384
  _SUBHTTPHEADER._serialized_start=386
  _SUBHTTPHEADER._serialized_end=434
  _HTTPRESPONSE._serialized_start=437
  _HTTPRESPONSE._serialized_end=587
# @@protoc_insertion_point(module_scope)
