# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: eventbus.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x65ventbus.proto\x12\x0eKismetEventBus\"#\n\rEventbusEvent\x12\x12\n\nevent_json\x18\x01 \x02(\t\")\n\x18\x45ventbusRegisterListener\x12\r\n\x05\x65vent\x18\x01 \x03(\t\"F\n\x14\x45ventbusPublishEvent\x12\x12\n\nevent_type\x18\x01 \x02(\t\x12\x1a\n\x12\x65vent_content_json\x18\x02 \x02(\tB\x02H\x03')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'eventbus_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\003'
  _EVENTBUSEVENT._serialized_start=34
  _EVENTBUSEVENT._serialized_end=69
  _EVENTBUSREGISTERLISTENER._serialized_start=71
  _EVENTBUSREGISTERLISTENER._serialized_end=112
  _EVENTBUSPUBLISHEVENT._serialized_start=114
  _EVENTBUSPUBLISHEVENT._serialized_end=184
# @@protoc_insertion_point(module_scope)
