# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: linuxbluetooth.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import kismet_pb2 as kismet__pb2
import datasource_pb2 as datasource__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14linuxbluetooth.proto\x12\x14KismetLinuxBluetooth\x1a\x0ckismet.proto\x1a\x10\x64\x61tasource.proto\"\x8f\x01\n\x17SubLinuxBluetoothDevice\x12\x10\n\x08time_sec\x18\x01 \x02(\x04\x12\x11\n\ttime_usec\x18\x02 \x02(\x04\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x02(\t\x12\x0c\n\x04name\x18\x04 \x02(\t\x12\x0f\n\x07txpower\x18\x05 \x02(\x05\x12\x0c\n\x04type\x18\x06 \x02(\x05\x12\x11\n\tuuid_list\x18\x07 \x03(\t\"\xf0\x01\n\x18LinuxBluetoothDataReport\x12%\n\x03gps\x18\x01 \x01(\x0b\x32\x18.KismetDatasource.SubGps\x12.\n\x07message\x18\x02 \x01(\x0b\x32\x1d.KismetExternal.MsgbusMessage\x12\x0f\n\x07warning\x18\x03 \x01(\t\x12+\n\x06signal\x18\x04 \x01(\x0b\x32\x1b.KismetDatasource.SubSignal\x12?\n\x08\x62tdevice\x18\x05 \x02(\x0b\x32-.KismetLinuxBluetooth.SubLinuxBluetoothDeviceB\x02H\x03')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'linuxbluetooth_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\003'
  _SUBLINUXBLUETOOTHDEVICE._serialized_start=79
  _SUBLINUXBLUETOOTHDEVICE._serialized_end=222
  _LINUXBLUETOOTHDATAREPORT._serialized_start=225
  _LINUXBLUETOOTHDATAREPORT._serialized_end=465
# @@protoc_insertion_point(module_scope)
