html, body {
    height: 100%
}

body {
    overflow: hidden;
}

#mainholder {
    /*
    min-height: 100%;
    position: relative;
    */

    min-width: 100%;
    min-height: 100%;
    display: -webkit-box;
    display: -moz-box;
    display: -ms-flexbox;
    display: -webkit-flex;
    display: flex;
    -ms-flex-direction: column;
    -moz-flex-direction: column;
    -webkit-flex-direction: column;
    flex-direction: column;

}

#panelcontainer {
    /*padding-bottom: 30px;*/

    -webkit-box-flex: 1; /* OLD - iOS 6-, Safari 3.1-6 */
    -moz-box-flex: 1; /* OLD - Firefox 19- */
    -webkit-flex: 1; /* Chrome */
    -ms-flex: 1; /* IE 10 */
    flex: 1; /* NEW, */
}

#golden {
    width: 100%;
    height: calc(100% - 60px);
}

#creditbar {

    /*
    width:100%;
    position: absolute;
    left: 0;
    bottom: 30px; 
    */

    background-color: #cccccc;
    font-size: 13px;
    height: 20px;
}


.ui-layout-pane-north {
    padding: 0 !important;
    border-bottom: none;
    overflow: hidden !important;
}

.ui-layout-center {
    overflow: hidden !important;
}

.ui-layout-pane-south {
    padding: 0 !important;
    border-top: none;
}

#topbar {
    position: relative;
    overflow: hidden !important;
    height: 40px;
    background-color: #CCCCCC;
    padding-left: 10px;
    padding-right: 10px;
}

/* Title text */
#topbar_title {
    float: left;

    position: relative;
    padding-left: 10px;

    top: 50%;
    transform: translateY(-50%);

    font-style: bold;
    font-size: 24px;
}

/* Sidebar button */
#pm_open {
    top: 50%;
    transform: translateY(-50%);

    margin-left: 10px;
    margin-right: 10px;

    float: left;

    position: relative;

    font-size: 18px;
}

div.topright_holder {
    float: right;
    position: relative;
    top: 50%;
    transform: translateY(-50%);
    margin-right: 10px;
}

#topbar_battery {
    display: inline-block;
}

/* Tabs minimalism from http://keith-wood.name/uitabs.html#tabs-min */
.tabs-min { 
    background: transparent; 
    border: none; 
} 

.ui-tabs {
    padding-top: 0;
    padding-bottom: 0;
    padding-left: 0.5em;
    padding-right: 0.5em;
}

.ui-tabs-panel {
    padding: 0 !important;
}

.ui-widget-header { 
    background: transparent; 
    border: none; 
    border-bottom: 1px solid #909090; 
    -moz-border-radius: 0px; 
    -webkit-border-radius: 0px; 
    border-radius: 0px; 
} 
.ui-tabs-nav .ui-state-default { 
    background: transparent; 
    border: none; 
} 
.ui-tabs-nav .ui-state-active { 
    background: transparent url(images/uiTabsArrow.png) no-repeat bottom center; 
    border: none; 
} 
.ui-tabs-nav .ui-state-default a { 
    color: #808080; 
} 
.ui-tabs-nav .ui-state-active a { 
    color: #303030; 
}

.kismet-highlight {
    filter: brightness(0.75);
}

div.k-c-p-column {
    padding: 2px;
    margin: 3px;
    border-style: solid;
    border-width: 1px;
    background-color: #D0D0D0;
}

div.k-c-p-header {
    padding: 2px;
    margin: 3px;
    border-width: 1px;
    border-style: transparent;
}

div.k-c-p-column:hover {
    background-color: #B0B0B0;
}

i.k-c-p-c-mover {
    padding-left: 5px;
    padding-right: 5px;
    color: grey;
}

.k-c-p-c-enable {
    display: inline-block;
    width: 1em;
    padding-left: 5px;
    padding-right: 5px;
    text-align: center;
}

div.k-c-p-c-name {
    display: inline-block;
    margin: 3px;
    width: 15em;
}

div.k-c-p-c-title {
    display: inline-block;
    margin: 3px;
    width: 10em;
}

div.k-c-p-c-notes {
    display: inline-block;
    margin: 3px;
    width: 10em;
}

.toggle.expander.expanded {
    padding-left: 13px;
    background-position: left center;
    background-repeat: no-repeat;
    background-image: url(../images/expanded.gif);
}

.toggle.expander.collapsed {
    padding-left: 13px;
    background-position: left center;
    background-repeat: no-repeat;
    background-image: url(../images/collapsed.gif);
}        

.spacer {
    height: 5px;
    clear: both;
    width: 100%;
    display: block;
}

.fixeddt {
    table-layout: fixed;
    word-wrap: break-word;
}

.dt_td {
    text-overflow: clip;
    overflow: hidden;
    padding-right: 1px;
}

