# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kismet.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0ckismet.proto\x12\x0eKismetExternal\":\n\x07\x43ommand\x12\x0f\n\x07\x63ommand\x18\x01 \x02(\t\x12\r\n\x05seqno\x18\x02 \x02(\r\x12\x0f\n\x07\x63ontent\x18\x03 \x02(\x0c\"\xa1\x01\n\rMsgbusMessage\x12:\n\x07msgtype\x18\x01 \x02(\x0e\x32).KismetExternal.MsgbusMessage.MessageType\x12\x0f\n\x07msgtext\x18\x02 \x02(\t\"C\n\x0bMessageType\x12\t\n\x05\x44\x45\x42UG\x10\x01\x12\x08\n\x04INFO\x10\x02\x12\t\n\x05\x45RROR\x10\x04\x12\t\n\x05\x41LERT\x10\x08\x12\t\n\x05\x46\x41TAL\x10\x10\"\x06\n\x04Ping\"\x1a\n\x04Pong\x12\x12\n\nping_seqno\x18\x01 \x02(\r\"\"\n\x10\x45xternalShutdown\x12\x0e\n\x06reason\x18\x01 \x02(\t\"#\n\x0eSystemRegister\x12\x11\n\tsubsystem\x18\x01 \x02(\tB\x02H\x03')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'kismet_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\003'
  _COMMAND._serialized_start=32
  _COMMAND._serialized_end=90
  _MSGBUSMESSAGE._serialized_start=93
  _MSGBUSMESSAGE._serialized_end=254
  _MSGBUSMESSAGE_MESSAGETYPE._serialized_start=187
  _MSGBUSMESSAGE_MESSAGETYPE._serialized_end=254
  _PING._serialized_start=256
  _PING._serialized_end=262
  _PONG._serialized_start=264
  _PONG._serialized_end=290
  _EXTERNALSHUTDOWN._serialized_start=292
  _EXTERNALSHUTDOWN._serialized_end=326
  _SYSTEMREGISTER._serialized_start=328
  _SYSTEMREGISTER._serialized_end=363
# @@protoc_insertion_point(module_scope)
